package cn.com.qmth.stmms.biz.report.dao;

import cn.com.qmth.stmms.biz.report.model.ReportSubject;

import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;

import java.util.List;

/**
 * 课程总量分析 Dao
 *
 * @author xuwenjin
 * @date 2019-04-12
 */
public interface ReportSubjectDao
        extends PagingAndSortingRepository<ReportSubject, Integer>, JpaSpecificationExecutor<ReportSubject> {

    @Modifying
    @Query("delete from ReportSubject r where r.examId=?1 ")
    void deleteByExamId(Integer exmaId);

    /**
     * 查询考试数据
     *
     * @param examId
     * @return
     */
    @Query(value = "SELECT  s.examId," +
            "s.subjectCode," +
            "s.subjectName," +
            "count(s) ," +
            "sum( CASE WHEN s.absent = true THEN 1 ELSE 0 END ) ," +
            "sum( CASE WHEN s.breach = true THEN 1 ELSE 0 END ) ," +
            "count(s) - sum( CASE WHEN s.absent = true THEN 1 ELSE 0 END )  ," +
            "max(s.objectiveScore + s.subjectiveScore) ," +
            "min(s.objectiveScore + s.subjectiveScore) ," +
            "avg(s.objectiveScore + s.subjectiveScore) ," +
            "sum( CASE WHEN (s.objectiveScore + s.subjectiveScore) >= e.passScore  THEN 1 ELSE 0 END )  " +
            "FROM " +
            "ExamStudent s , Exam e where s.examId = e.id " +
            "and s.examId = ?1  GROUP BY s.subjectCode, s.subjectName")
    List<Object[]> queryReportSubjectData(Integer examId);

    /**
     * 查询课程平均分
     *
     * @param examId
     * @param subjectCode
     */
    @Query(value = "select avg(s.objectiveScore + s.subjectiveScore) " +
            "from ExamStudent s where s.examId = ?1 and s.subjectCode = ?2 ")
    Double querySubjectAvgScore(Integer examId, String subjectCode);

}
