package cn.com.qmth.stmms.biz.report.service.impl;

import java.util.LinkedList;
import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.report.dao.ReportSubjectDao;
import cn.com.qmth.stmms.biz.report.model.ReportSubject;
import cn.com.qmth.stmms.biz.report.query.ReportSubjectQuery;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectService;
import cn.com.qmth.stmms.biz.utils.DoubleUtil;

/**
 * 课程总量统计service
 *
 * @author xuwenjin
 * @Date 2019-04-12 18:10
 */
@Service
public class ReportSubjectServiceImpl extends BaseQueryService<ReportSubject> implements ReportSubjectService {

    @Autowired
    private ReportSubjectDao reportSubjectDao;

    @Override
    public ReportSubject save(ReportSubject reportSubject) {
        return reportSubjectDao.save(reportSubject);
    }

    @Transactional
    @Override
    public void deleteByExamId(Integer examId) {
        reportSubjectDao.deleteByExamId(examId);
    }

    @Override
    public ReportSubjectQuery findByQuery(final ReportSubjectQuery query) {
        checkQuery(query);
        Page<ReportSubject> result = reportSubjectDao.findAll(new Specification<ReportSubject>() {

            @Override
            public Predicate toPredicate(Root<ReportSubject> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();

                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                return predicates.isEmpty() ? cb.conjunction()
                        : cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        }, query);
        fillResult(result, query);

        return query;
    }

    @Override
    public void saveReportSubjectData(Integer examId) {
        List<Object[]> reportSubjectList = reportSubjectDao.queryReportSubjectData(examId);
        for (Object[] rs : reportSubjectList) {
            ReportSubject subject = new ReportSubject();
            subject.setExamId(Integer.parseInt(rs[0].toString()));
            subject.setSubjectCode(rs[1].toString());
            subject.setSubjectName(rs[2].toString());
            subject.setTotalCount(Integer.parseInt(rs[3].toString()));
            subject.setAbsentCount(Integer.parseInt(rs[4].toString()));
            subject.setBreachCount(Integer.parseInt(rs[5].toString()));
            subject.setRealityCount(Integer.parseInt(rs[6].toString()));
            subject.setMaxScore(Double.parseDouble(rs[7].toString()));
            subject.setMinScore(Double.parseDouble(rs[8].toString()));
            subject.setAvgScore(DoubleUtil.format(Double.parseDouble(rs[9].toString()), "0.00"));
            subject.setPassCount(Integer.parseInt(rs[10].toString()));
            subject.setPassRate(DoubleUtil.div(Integer.parseInt(rs[10].toString()), Integer.parseInt(rs[6].toString()), 2));
            reportSubjectDao.save(subject);
        }
    }

}
