package cn.com.qmth.stmms.biz.report.utils;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectClassModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectCollegeModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectGroupModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectQuestionModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectRangeModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectTeacherClassModule;
import cn.com.qmth.stmms.biz.report.utils.module.SubjectTeacherModule;

public class ReportContext {

    private Exam exam;

    private List<Module> modules;
    
    private Map<String,String> subjectMap;

    public ReportContext(Exam exam) {
        this.exam = exam;
        this.modules = new LinkedList<Module>();

        this.modules.add(new SubjectRangeModule(this));
        this.modules.add(new SubjectModule(this));
        this.modules.add(new SubjectClassModule(this));
        this.modules.add(new SubjectCollegeModule(this));
        this.modules.add(new SubjectTeacherModule(this));
        this.modules.add(new SubjectTeacherClassModule(this));
        this.modules.add(new SubjectQuestionModule(this));
        this.modules.add(new SubjectGroupModule(this));
        
        this.subjectMap = new HashMap<String, String>();
    }

    public void process(ExamStudent student) {
        for (Module module : modules) {
            subjectMap.put(student.getSubjectCode(), student.getSubjectName());
            module.process(student);
        }
    }

    public void save() {
        for (Module module : modules) {
            module.save();
        }
    }

    public Double getPassScore() {
        return exam.getPassScore();
    }

    public Double getExcellentScore() {
        return exam.getExcellentScore();
    }

    public double getRangeInterval() {
        return 1;
    }
    
    public Integer getExamId() {
        return exam.getId();
    }

    public String getSubjectName(String subjectCode){
        return subjectMap.get(subjectCode);
    }
}
